package com.huoran.iasf.common.aop.aspect;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.entity.SysRole;
import com.huoran.iasf.entity.SysUser;
import com.huoran.iasf.service.HttpSessionService;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 概念：最终控制列表中显示哪些 部门/人 创建的数据
 * 1、需要数据权限控制的列表， 需要有创建人字段， 示例：文章管理
 * 2、配置角色的数据范围（本部门，其他部门等）， 以及绑定的部门
 * 3、加个注解，用来查询当前等路人的多个角色（并集）， 根据角色数据范围， 获取绑定的部门id， 查关联的用户id
 * 4、在查某个模块的list或page的时候，手动queryWrapper.in(createId, 关联的用户id)
 *
 * @author cheney
 */
@Aspect
@Component
public class DataScopeAspect {

    @Resource
    HttpSessionService sessionService;
    @Resource
    com.huoran.iasf.service.RoleService roleService;
    @Resource
    com.huoran.iasf.service.DeptService deptService;
    @Resource
    com.huoran.iasf.service.UserService userService;


    @Pointcut("@annotation(com.huoran.iasf.common.aop.annotation.DataScope)")
    public void dataScopePointCut() {
    }

    @Before("dataScopePointCut()")
    public void doBefore(JoinPoint point) {
        handleDataScope(point);
    }

    protected void handleDataScope(final JoinPoint joinPoint) {
        //获取当前登陆人
        Integer id = sessionService.getCurrentUserId();
        //获取当前登陆人角色, 如果无角色, 那么不限制
        List<SysRole> sysRoles = roleService.getRoleInfoByUserId(id);
        if (CollectionUtils.isEmpty(sysRoles) || sysRoles.size() == 0) {
            return;
        }
        //角色未配置数据权限范围, 那么不限制
        List<SysRole> list = sysRoles.parallelStream().filter(one -> null != one.getDataScope()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list) || list.size() == 0) {
            return;
        }
        //如果存在某角色配置了全部范围， 那么不限制
        if (list.stream().anyMatch(sysRole -> Constant.DATA_SCOPE_ALL.equals(sysRole.getDataScope()))) {
            return;
        }
        //获取绑定的人
        List<String> userIds = this.getUserIdsByRoles(list, id);
        Object params = joinPoint.getArgs()[0];
        if (params instanceof com.huoran.iasf.entity.BaseEntity) {
            com.huoran.iasf.entity.BaseEntity baseEntity = (com.huoran.iasf.entity.BaseEntity) params;
            baseEntity.setCreateIds(userIds);
        }

    }

    /**
     * 获取最终的用户id
     *
     * @param sysRoles 角色
     * @param userId   当前用户id
     * @return 用户id集合
     */
    private List<String> getUserIdsByRoles(List<SysRole> sysRoles, Integer userId) {
        //本人
        SysUser sysUser = userService.getById(userId);
        //本部门
        // SysDept sysDept = deptService.getById(sysUser.getDeptId());
        //部门ids， 定义哪些部门最终拥有权限查看
        LinkedList<Object> deptList = new LinkedList<>();
        //用户ids，定义列表中哪些人创建的可查看
        LinkedList<Object> userIdList = new LinkedList<>();
        //根据数据权限范围分组， 不同的数据范围不同的逻辑处理
        /*Map<Integer, List<SysRole>> dataScopeMap = sysRoles.parallelStream().collect(Collectors.groupingBy(com.huoran.iasf.entity.SysRole::getDataScope));
        dataScopeMap.forEach((k, v) -> {
            if (Constant.DATA_SCOPE_CUSTOM.equals(k)) {
                //自定义
                //根据角色id，获取所有自定义关联的部门id
                QueryWrapper<com.huoran.iasf.entity.SysRoleDeptEntity> queryWrapper = Wrappers.<com.huoran.iasf.entity.SysRoleDeptEntity>query().select("dept_id").in("role_id", v.parallelStream().map(com.huoran.iasf.entity.SysRole::getId).collect(Collectors.toList()));
                deptList.addAll(sysRoleDeptService.listObjs(queryWrapper));
            } else if (Constant.DATA_SCOPE_DEPT_AND_CHILD.equals(k)) {
                //本部门及以下
                if (sysDept != null && StringUtils.isNotBlank(sysDept.getDeptNo())) {
                    //获取本部门以下所有关联的部门
                    QueryWrapper<com.huoran.iasf.entity.SysDept> queryWrapper = Wrappers.<com.huoran.iasf.entity.SysDept>query().select("id").like("relation_code", sysDept.getDeptNo());
                    deptList.addAll(deptService.listObjs(queryWrapper));
                }
            } else if (Constant.DATA_SCOPE_DEPT.equals(k)) {
                //本部门
                if (sysDept != null && StringUtils.isNotBlank(sysDept.getId())) {
                    deptList.add(sysDept.getId());
                }
            } else if (Constant.DATA_SCOPE_DEPT_SELF.equals(k)) {
                //自己
                userIdList.add(userId);
            }
        });*/
        if (!CollectionUtils.isEmpty(deptList)) {
            QueryWrapper<com.huoran.iasf.entity.SysUser> queryWrapper = Wrappers.<com.huoran.iasf.entity.SysUser>query().select("id").in("dept_id", deptList);
            userIdList.addAll(userService.listObjs(queryWrapper));
        }
        //如果配置了角色数据范围， 最终没有查到userId， 那么返回无数据
        if (CollectionUtils.isEmpty(userIdList)) {
            throw new BusinessException(BaseResponseCode.NOT_ACCOUNT);
        }
        return userIdList.parallelStream().map(Object::toString).collect(Collectors.toList());
    }
}
