package com.huoran.iasf.common.exception.handler;

import com.alibaba.fastjson.JSONException;
import com.google.common.collect.Maps;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.EncryptRequestException;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.UnauthorizedException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.R;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authz.AuthorizationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;
import java.util.HashMap;
import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;

/**
 * RestExceptionHandler
 * 全局异常拦截器
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Slf4j
@RestControllerAdvice
public class RestExceptionHandler {

    /**
     * 系统繁忙，请稍候再试"
     */
    @ExceptionHandler(Exception.class)
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public R handleException(Exception e) {
        log.error("Exception,exception:{}", e, e);
        return R.getResult(BaseResponseCode.SYSTEM_BUSY);
    }

    @ExceptionHandler(IllegalArgumentException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public R illegalArgumentException(IllegalArgumentException e) {
        log.error("Exception,exception:{}", e, e);
        return R.getResult(BaseResponseCode.METHOD_ARGUMENT_NOT_VALID_EXCEPTION);
    }

    @ExceptionHandler(EncryptRequestException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public R encryptRequestException(EncryptRequestException e) {
        log.error("Exception,exception:{}", e, e);
        return R.getResult(BaseResponseCode.METHOD_ARGUMENT_NOT_VALID_EXCEPTION);
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public R illegalArgumentException(JSONException e) {
        log.error("Exception,exception:{}", e, e);
        return R.getResult(BaseResponseCode.ILLEGAL_CHARACTERS);
    }

    @ExceptionHandler(AuthenticationException.class)
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public R authenticationException(AuthenticationException e) {
        log.error("Exception,exception:{}", e, e);
//        throw new BusinessException(BaseResponseCode.TOKEN_ERROR);
        return new R(BaseResponseCode.TOKEN_ERROR.getCode(), BaseResponseCode.TOKEN_ERROR.getMsg());
    }

    /**
     * 自定义全局异常处理
     */
    @ExceptionHandler(value = BusinessException.class)
    @ResponseStatus(HttpStatus.OK)
    public R businessExceptionHandler(BusinessException e) {
        log.error("Exception,exception:{}", e, e);
        BaseResponseCode em = e.getBaseResponseCode();
        return new R(em.getCode(), em.getMsg());
    }

    /**
     * 权限全局异常处理
     */
    @ExceptionHandler(value = UnauthorizedException.class)
//    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public R businessExceptionHandler(UnauthorizedException e) {
        log.error("Exception,exception:{}", e, e);
        BaseResponseCode em = e.getBaseResponseCode();
        return new R(em.getCode(), em.getMsg());
    }

    @ExceptionHandler(value = NotFoundException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public R businessExceptionHandler(NotFoundException e) {
        log.error("Exception,exception:{}", e, e);
        BaseResponseCode em = e.getBaseResponseCode();
        return new R(em.getCode(), em.getMsg());
    }

    /**
     * 没有权限 返回403视图
     */
    @ExceptionHandler(value = AuthorizationException.class)
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public R errorPermission(AuthorizationException e) {
        log.error("Exception,exception:{}", e, e);
        return new R(BaseResponseCode.UNAUTHORIZED_ERROR);

    }

    /**
     * 处理参数格式校验异常
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.METHOD_NOT_ALLOWED)
    public R handleValidException(MethodArgumentNotValidException e){
        log.error("参数格式校验异常");
        BindingResult bindingResult = e.getBindingResult();
        HashMap<String, String> errorMap = Maps.newHashMap();
        AtomicReference<String> errorMsg = new AtomicReference<>("");
        bindingResult.getFieldErrors().forEach(fieldError -> {
            errorMsg.set(fieldError.getDefaultMessage());
            errorMap.put(fieldError.getField(),fieldError.getDefaultMessage());
        });
        return R.fail(errorMsg);
    }

    /**
     * 校验List<entity>类型， 需要controller添加@Validated注解
     * 处理Validated List<entity> 异常
     */
    @ExceptionHandler
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public R handle(ConstraintViolationException exception) {
        log.error("methodArgumentNotValidExceptionHandler bindingResult.allErrors():{},exception:{}", exception, exception);
        Set<ConstraintViolation<?>> violations = exception.getConstraintViolations();
        StringBuilder builder = new StringBuilder();
        for (ConstraintViolation<?> violation : violations) {
            builder.append(violation.getMessage());
            break;
        }
        return R.getResult(BaseResponseCode.METHOD_ARGUMENT_NOT_VALID_EXCEPTION.getCode(), builder.toString());
    }

    @ExceptionHandler
    @ResponseStatus(HttpStatus.NOT_FOUND)
    public R handle(ValidationException e) {
        log.error("Exception,exception:{}", e, e);
        return new R(404, e.getMessage());
    }

}
