package com.huoran.iasf.common.filter;


import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.UnauthorizedException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;

import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

import static com.huoran.iasf.service.HttpApiSessionService.*;

/**
 * shiro过滤器 登录用户权限认证授权
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Slf4j
@WebFilter(filterName = "tokenFilter", urlPatterns = "/**")
@Order(1)
//@Component
public class AuthFilter implements Filter {
    /**
     * 白名单
     */
    private static final String[] whiteList =
            {"/iasf/sys/user/login",
            "/iasf/sys/exportFailure","/iasf/sys/getVerify","/iasf/doc.html"};

    //需要拦截的地址,验证token是否存在及有效
    @Override
    @SneakyThrows
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)  {
        //每一次的请求先校验cookie

        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse resp = (HttpServletResponse) response;
        resp.setHeader("Access-Control-Allow-Origin", req.getHeader("Origin"));
        resp.setHeader("Access-Control-Allow-Methods", req.getMethod());
        resp.setHeader("Access-Control-Max-Age", "3600");
        resp.setHeader("Access-Control-Allow-Headers", req.getHeader("Access-Control-Request-Headers"));
        String url = req.getRequestURI();
        log.info("url:{}", url);

        if (Arrays.asList(whiteList).contains(url)) {
            chain.doFilter(request, response);
        } else {
            //拦截接口
            //从header中获取token
            String token = req.getHeader("token");
            //如果header中不存在token，则从参数中获取token
            if (StringUtils.isBlank(token)) {
                token = request.getParameter("token");
            }
            //token为空返回
            if (StringUtils.isBlank(token)) {
                resp.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                resp.setCharacterEncoding("utf-8");
                resp.setContentType(MediaType.APPLICATION_JSON_VALUE);
                request.setAttribute("filterError", new UnauthorizedException(BaseResponseCode.TOKEN_ERROR));
                // 指定处理该请求的处理器
                request.getRequestDispatcher(Constant.ERROR_CONTROLLER_PATH).forward(request, resp);
            }
            //  校验并解析token，如果token过期或者篡改，则会返回null
            Claims claims = checkJWT(token);
            if (null == claims) {
                resp.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                resp.setCharacterEncoding("utf-8");
                resp.setContentType(MediaType.APPLICATION_JSON_VALUE);
                request.setAttribute("filterError", new UnauthorizedException(BaseResponseCode.TOKEN_ERROR));
                // 指定处理该请求的处理器
                request.getRequestDispatcher(Constant.ERROR_CONTROLLER_PATH).forward(request, resp);
            }else {
                //  校验通过后，设置用户信息到request里，在Controller中从Request域中获取用户信息
                request.setAttribute(USER_ID_KEY, claims.get(USER_ID_KEY));
                request.setAttribute(USER_USERNAME_KEY, claims.get(USER_USERNAME_KEY));
            }
            chain.doFilter(request, response);
        }

    }

    /**
     * 校验token
     */
    public Claims checkJWT(String token) {

        try {
            final Claims claims = Jwts.parser().setSigningKey(APP_SECRET).
                    parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
            return null;
        }
    }

}
