package com.huoran.iasf.common.filter;

import com.alibaba.fastjson.JSONObject;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.safety.Safelist;

import javax.servlet.ReadListener;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.regex.Pattern;

@Slf4j
public class XssHttpServletRequestWrapper extends HttpServletRequestWrapper {

    /**
     * post请求体
     */
    private byte[] body;

    /**
     * 是否是文件上传
     */
    private boolean fileUpload = true;

    //富文本验证链接
    private static final String[] whiteList =
            {"/iasf/sysContent/save",
                    "/iasf/sysContent/update"};

    /**
     * sql注入正则
     */
    private static String badStrReg =
            "\\b(and|or)\\b.{1,6}?(=|>|<|\\bin\\b|\\blike\\b)|\\/\\*.+?\\*\\/|<\\s*script\\b|\\bEXEC\\b|UNION.+?SELECT|UPDATE.+?SET|INSERT\\s+INTO.+?VALUES|(SELECT|DELETE).+?FROM|(CREATE|ALTER|DROP|TRUNCATE)\\s+(TABLE|DATABASE)";

    /**
     * xss脚本正则
     */
    private final static Pattern[] scriptPatterns = {
            Pattern.compile("<script>(.*?)</script>", Pattern.CASE_INSENSITIVE),
            Pattern.compile("src[\r\n]*=[\r\n]*\\\'(.*?)\\\'", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL),
            Pattern.compile("</script>", Pattern.CASE_INSENSITIVE),
            Pattern.compile("<script(.*?)>", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL),
            Pattern.compile("eval\\((.*?)\\)", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL),
            Pattern.compile("expression\\((.*?)\\)", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL),
            Pattern.compile("javascript:", Pattern.CASE_INSENSITIVE),
            Pattern.compile("script", Pattern.CASE_INSENSITIVE),
            Pattern.compile("ScriPT", Pattern.CASE_INSENSITIVE),
            Pattern.compile("meta", Pattern.CASE_INSENSITIVE),
            Pattern.compile("vbscript:", Pattern.CASE_INSENSITIVE),
            Pattern.compile("onload(.*?)=", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL)
    };

    public XssHttpServletRequestWrapper() {
        super(null);
    }

    /**
     * 构造函数 - 获取post请求体
     * @param httpservletrequest
     * @throws IOException
     */
    public XssHttpServletRequestWrapper(HttpServletRequest httpservletrequest) throws IOException {
        super(httpservletrequest);
        String sessionStream = getBodyString(httpservletrequest);
        body = sessionStream.getBytes(StandardCharsets.UTF_8);
        System.out.println(httpservletrequest.getRequestURI());
        if(Arrays.asList(whiteList).contains(httpservletrequest.getRequestURI())){
            fileUpload = false;
        }
    }

    /**
     * 读取post请求体
     * @param httpservletrequest
     * @return
     * @throws IOException
     */
    private String getBodyString(HttpServletRequest httpservletrequest) throws IOException {
        StringBuilder sb = new StringBuilder();
        InputStream ins = httpservletrequest.getInputStream();
        try (BufferedReader isr = new BufferedReader(new InputStreamReader(ins, StandardCharsets.UTF_8));) {
            String line = "";
            while ((line = isr.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            throw e;
        }
        return sb.toString();
    }

    /**
     * 过滤springmvc中的 @RequestParam 注解中的参数
     * @param s
     * @return
     */
    @Override
    public String[] getParameterValues(String s) {
        String[] str = super.getParameterValues(s);
        if (str == null) {
            return null;
        }
        int i = str.length;
        String[] as1 = new String[i];
        for (int j = 0; j < i; j++) {
            as1[j] = cleanXSS(cleanSQLInject(str[j]));
        }
//        log.info("XssHttpServletRequestWrapper净化后的请求为：========== {}", Arrays.toString(as1));
        return as1;
    }

    /**
     * 过滤request.getParameter的参数
     * @param s
     * @return
     */
    @Override
    public String getParameter(String s) {
        String s1 = super.getParameter(s);
        if (s1 == null) {
            return null;
        } else {
            String s2 = cleanXSS(cleanSQLInject(s1));
//            log.info("XssHttpServletRequestWrapper净化后的请求为：========== {}",  s2);
            return s2;
        }
    }


    /**
     * 过滤请求体 json 格式的
     * @return
     * @throws IOException
     */
    @Override
    public ServletInputStream getInputStream() throws IOException {
        // 非文件上传进行过滤
        if (!fileUpload) {

            try {
                // 解析请求体为字符串
                String bodyStr = new String(body, StandardCharsets.UTF_8);

                // 清理HTML，只允许安全的元素和属性
                Safelist safelist = Safelist.basicWithImages(); // 自定义safelist
                String safeHtml = Jsoup.clean(bodyStr, "", safelist, new Document.OutputSettings().prettyPrint(false));
                cleanSQLInject(safeHtml);

            } catch (Exception e) {
                // 处理解析或处理过程中的任何异常
                log.error("Error processing request body {}",  e.getMessage());
            }
        }
        // 将请求体参数流转 -- 流读取一次就会消失,所以我们事先读取之后就存在byte数组里边方便流转
        final ByteArrayInputStream bais = new ByteArrayInputStream(body);
        return new ServletInputStream() {

            @Override
            public int read() throws IOException {
                return bais.read();
            }

            @Override
            public boolean isFinished() {
                return false;
            }

            @Override
            public boolean isReady() {
                return false;
            }

            @Override
            public void setReadListener(ReadListener readListener) {
            }
        };
    }


    public String cleanXSS(String src) {
        if (StringUtils.isBlank(src)) {
            return src;
        }

        // 创建一个允许的HTML标签和属性的Safelist
        Safelist safelist = Safelist.relaxed() // 允许大多数基本的HTML标签和属性
                .addTags("img") // 添加额外的标签，如img（记得添加允许的属性，如src和alt）
                .addAttributes("*", "class") // 允许所有标签使用"class"属性
                .addAttributes("img", "src", "alt") // 允许img标签的src和alt属性
                .addProtocols("img", "src", "http", "https") // 只允许http和https协议的src
                ; // 移除协议相对URL，避免安全问题

        // 使用JSoup进行清理
        Document document = Jsoup.parseBodyFragment(src, ""); // 解析HTML片段
        document.outputSettings(new Document.OutputSettings().prettyPrint(false)); // 禁止美化输出，保持原始结构
        String html = document.html();
        String clean = Jsoup.clean(html, "", safelist);// 使用Safelist进行清理

        return clean; // 返回清理后的HTML字符串
    }

    /**
     * 清除xss
     * @param src 单个参数
     * @return
     */
    /*public String cleanXSS(String src) {
        if(StringUtils.isBlank(src)){
            return src;
        }
        String temp = src;
        // 校验xss脚本
        for (Pattern pattern : scriptPatterns) {
            temp = pattern.matcher(temp).replaceAll("");
        }
        // 校验xss特殊字符 匹配一个空白字符（包括空格、制表符、换页符和换行符等）//这个可以不用，因为有写数据用富文本编辑的时候有换行
//        temp = temp.replaceAll("\0|\n|\r", "");
        temp = temp.replaceAll("<", "<").replaceAll(">", ">");

        if (!temp.equals(src)) {

            log.error("xss攻击检查：参数含有非法攻击字符，已禁止继续访问！");
            log.error("原始输入信息-->" + temp);

            throw new BusinessException(BaseResponseCode.XSS_FILTER);
        }

        return src;
    }*/

    /**
     * 过滤sql注入 -- 需要增加通配，过滤大小写组合
     * @param src 单个参数值
     * @return
     */
    public String cleanSQLInject(String src) {
        if(StringUtils.isBlank(src)){
            return src;
        }
        String cleanedText = Jsoup.clean(src, Safelist.basic());

        String SQL_KEYWORD_PATTERN =
                "(?i)(?:(?!<[^>]*?>))((select|update|insert|delete|drop|create|alter|exec|union|table|database)[^a-zA-Z0-9])";

        // 过滤SQL关键字
        cleanedText = cleanedText.replaceAll(SQL_KEYWORD_PATTERN, "");
        // 非法sql注入正则
//        Pattern sqlPattern = Pattern.compile(badStrReg, Pattern.CASE_INSENSITIVE);
//        if (sqlPattern.matcher(src.toLowerCase()).find()) {
//            log.error("sql注入检查：输入信息存在SQL攻击！");
//            throw new BusinessException(BaseResponseCode.SQL_FILTER);
//        }
        return cleanedText;
    }
}