package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.LogAnnotation;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysRole;
import com.huoran.iasf.entity.SysRolePermission;
import com.huoran.iasf.service.HttpSessionService;
import com.huoran.iasf.service.RolePermissionService;
import com.huoran.iasf.service.RoleService;
import com.huoran.iasf.vo.req.RolePermissionOperationReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 角色管理
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@RequestMapping("/sys")
@RestController
@Api(tags = "角色管理")
public class RoleController {
    @Resource
    private RoleService roleService;
    @Resource
    private RolePermissionService rolePermissionService;
    @Resource
    private HttpSessionService httpSessionService;
    @NoRepeatSubmit
    @PostMapping("/addRole")
    @ApiOperation(value = "新增角色接口")
    @LogAnnotation(title = "角色管理", action = "新增角色")
    @RequiresPermissions("sys:role:add")
    public R addRole(@RequestBody @Valid SysRole vo) {
        boolean ok = roleService.checkDeptRepeat(vo);
        if (ok) {
            return R.fail("已存在相同的角色名称:" + vo.getName());
        }
        roleService.addRole(vo);
        //删除角色权限关联
        rolePermissionService.remove(Wrappers.<SysRolePermission>lambdaQuery().eq(SysRolePermission::getRoleId, vo.getId()));
        if (!CollectionUtils.isEmpty(vo.getPermissions())) {
            RolePermissionOperationReqVO reqVO = new RolePermissionOperationReqVO();
            reqVO.setRoleId(vo.getId());
            reqVO.setPermissionIds(vo.getPermissions());
            rolePermissionService.addRolePermission(reqVO);
            // 刷新权限
            httpSessionService.refreshRolePermission(vo.getId());
        }
        return R.success();
    }

    @PostMapping("/role/deleted")
    @ApiOperation(value = "删除角色接口")
    @Decrypt
    @LogAnnotation(title = "角色管理", action = "删除角色")
    @RequiresPermissions("sys:role:deleted")
    public R deleted(@RequestBody @ApiParam(value = "id集合") List<String> ids) {
        roleService.deletedRole(ids);
        return R.success();
    }

    @PostMapping("/updateRole")
    @ApiOperation(value = "更新角色信息接口")
    @LogAnnotation(title = "角色管理", action = "更新角色信息")
    @RequiresPermissions("sys:role:update")
    public R updateDept(@RequestBody SysRole vo) {
        if (StringUtils.isEmpty(vo.getId())) {
            return R.fail("id不能为空");
        }
        boolean ok = roleService.checkDeptRepeat(vo);
        if (ok) {
            return R.fail("已存在相同的角色名称:" + vo.getName());
        }
        roleService.updateRole(vo);
        return R.success();
    }

    @PostMapping("/role")
    @Decrypt
    @ApiOperation(value = "查询角色详情接口")
    @LogAnnotation(title = "角色管理", action = "查询角色详情")
    @RequiresPermissions("sys:role:detail")
    public R detailInfo(@RequestBody Integer id) {
        // return R.success(roleService.detailInfo(id));
        SysRole role = roleService.getById(id);
        QueryWrapper<SysRolePermission> wrapper = new QueryWrapper<>();
        wrapper.eq("role_id", id).select("permission_id");
        List<SysRolePermission> list = rolePermissionService.list(wrapper);
        List<Integer> permissionList = new ArrayList<>();
        list.forEach(rolePermission -> {
            permissionList.add(rolePermission.getPermissionId());
        });
        HashMap<String, Object> map = new HashMap<>();
        map.put("role", role);
        map.put("permissionList", permissionList);
        return R.success(map);
    }

    @PostMapping("/roles")
    @Decrypt
    @ApiOperation(value = "分页获取角色信息接口")
    @LogAnnotation(title = "角色管理", action = "分页获取角色信息")
    @RequiresPermissions("sys:role:list")
    @SuppressWarnings("unchecked")
    public R pageInfo(@RequestBody SysRole vo) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(vo.getName())) {
            //中文转码
            String name = URLDecoder.decode(vo.getName(), StandardCharsets.UTF_8.toString());
            vo.setName(name);
        }
        LambdaQueryWrapper<SysRole> queryWrapper = Wrappers.lambdaQuery();
        if (!StringUtils.isEmpty(vo.getName())) {
            queryWrapper.like(SysRole::getName, vo.getName());
        }
        queryWrapper.orderByDesc(SysRole::getCreateTime);
        return R.success(roleService.page(vo.getQueryPage(), queryWrapper));
    }

}
